module ShotClock_CU (

// ShotClock Control Unit

	// Inputs
	I$Clock,	// Master clock
	I$Reset,	// Master reset, active high
	I$Run,		// RUN, active high
	I$CountEqualsZero,	// Indicates counter has
						// reached zero, active high

	// Outputs
	O$Buzzer,	// BUZZER, active high
	O$EnableCounter	// Enables count-down counter

);

// Port modes
input I$Clock,I$Reset,I$Run,I$CountEqualsZero;
output O$Buzzer,O$EnableCounter;

// State definitions
parameter	NumStateBits = 4;
parameter	Idle	= 4'b0001;	// one-hot encoding for FPGAs
parameter	CountDown	= 4'b0010;
parameter	Pause	= 4'b0100;
parameter	Buzz	= 4'b1000;

// Registered variables
reg	[NumStateBits-1:0] State;	// Present-state register
reg	[NumStateBits-1:0] NextState;	// Next-state decoder output
reg O$Buzzer;
reg O$EnableCounter;


// Next-state decoder and output decoder
always @ (State or I$Run or I$CountEqualsZero) begin

	// Nominal outputs
	NextState <= Idle;
	O$Buzzer <= 0;
	O$EnableCounter <= 0;

	// Decoder outputs
	case (State)
		Idle:	// Wait for RUN signal to assert
			NextState <= (I$Run) ? CountDown : Idle;

		CountDown:	// Enable counter. If counter has timed
					// out, then select "Buzz" as the next
					// state, otherwise look at "Run" to 
					// determine if next state is CountDown
					// or Pause.
			begin
				O$EnableCounter <= 1;
				if (I$CountEqualsZero)
					NextState <= Buzz;
				else
					NextState <= (I$Run) ? CountDown : Pause;
			end

		Pause:	// Wait for RUN signal to assert
			NextState <= (I$Run) ? CountDown : Pause;

		Buzz:	// Enable buzzer and stay in this state
				// (need to apply async reset to exit)
			begin
				O$Buzzer <= 1;
				NextState <= Buzz;
			end

	endcase

end

// State register
always @ (posedge I$Clock or posedge I$Reset)
	if (I$Reset == 1)
		State <= Idle;
	else
		State <= NextState;


endmodule
